/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.situation;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans.
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class SituationEigenschaften {
	public final long startZeit;
	public final long dauer;
	public final SystemObject[] strassenSegment;
	public final long startOffset;
	public final long endOffset;

	SituationEigenschaften(final Data datenSatz) {
		startZeit = datenSatz.getItem("StartZeit").asTimeValue().getMillis();
		dauer = datenSatz.getItem("Dauer").asTimeValue().getMillis();
		startOffset = datenSatz.getItem("StartOffset").asUnscaledValue().longValue();
		endOffset = datenSatz.getItem("EndOffset").asUnscaledValue().longValue();

		final Array array = datenSatz.getItem("StraßenSegment").asArray();
		if (array != null) {
			strassenSegment = new SystemObject[array.getLength()];
			for (int i = 0; i < array.getLength(); i += 1) {
				final SystemObject obj = array.getItem(i).asReferenceValue().getSystemObject();
				strassenSegment[i] = obj;
			}
		} else {
			strassenSegment = new SystemObject[0];
		}
	}

	/**
	 * Beschreiben diese Situationseigenschaften eine gültige Situation?
	 * 
	 * @param aktuelleZeit
	 *            aktuelle Zeit
	 * @return true, wenn ddiese Situationseigenschaften eine gültige Situation
	 *         beschreiben, false sonst
	 */
	public boolean istGueltig(final long aktuelleZeit) {
		if (strassenSegment.length == 0) {
			return false;
		}
		if (dauer == 0) {
			return startZeit <= aktuelleZeit;
		}
		return startZeit + dauer >= aktuelleZeit;
	}

}
